"""The Python task-oriented project manager.

Usage:
    pyx 
    pyx <task> [<task_args>...]
"""

import docopt
import os
import inspect
import subprocess
from pyx import utils as pyxutl

PYX_PATH = os.path.dirname(os.path.abspath(inspect.getsourcefile(lambda: 0)))


def retrieve_task(name):
    local = os.path.join(os.getcwd(), ".pyx", "tasks", name + ".py")
    absolute = os.path.join(PYX_PATH, "tasks", name + ".py")

    if os.path.isfile(local):
        return local
    elif os.path.isfile(absolute):
        return absolute
    else:
        raise Exception("No such task: {}".format(name))


def execute_task(task_name, args):
    task = retrieve_task(task_name)
    subprocess.run(["python3", task] + args)


def print_command_list():
    tasks = pyxutl.get_tasks()
    print("Tasks:")
    pyxutl.print_list(tasks)


def main():
    args = docopt.docopt(__doc__)
    task_name = args['<task>']
    if task_name is not None:
        execute_task(args['<task>'], args['<task_args>'])
    else:
        print(__doc__)
        print_command_list()


if __name__ == '__main__':
    main()



"""Distribute your project to a PyPI repository

Usage:
    release
"""


import docopt
import subprocess
import sys
import shutil
import pyx
import pyx.utils as pyxutl


def twine_command(args):
    cmd = ["twine", "upload"]

    if 'repository' in args:
        cmd.extend(["--repository-url", args['repository']])

    if 'username' in args:
        cmd.extend(["-u", args['username']])

    if 'password' in args:
        cmd.extend(["-p", args["password"]])

    cmd.extend(["--skip-existing", "dist/*"])

    print(cmd)
    return cmd


def git_release():
    version = pyxutl.get_settings()['version']
    subprocess.run(["git", "tag", "-a", version])


def main():
    pyxutl.ensure_pyx()

    args = docopt.docopt(__doc__)

    project = pyxutl.get_project()
    pypi = project.pypi

    version = pyxutl.get_settings()['version']
    message = "The project will be released as version {}. Are you sure?"
    if not pyxutl.prompt(message.format(version)):
        sys.exit(0)

    shutil.rmtree("build", ignore_errors=True)
    if pyxutl.is_git():
        git_release()
    subprocess.run(["python3", "setup.py", "sdist", "bdist_wheel"])
    subprocess.run(twine_command(pypi))


if __name__ == '__main__':
    main()



"""Pyx project creation

Usage:
    new <name>
"""

import os
import docopt
import sys
import getpass
from pyx import utils as pyxutl
from pyx import errors as pyxerr



TEST_SKELETON = '''"""Unit tests for {app}
"""

import unittest

class Test{app}(unittest.TestCase):
    def test_the_truth(self):
        assert True
'''


SETUP_SKELETON = '''import setuptools


settings = {{
    'name':     "{project}",
    'version':  "1.0",
    'author':   "{user}"
}}

if __name__ == '__main__':
    setuptools.setup(**settings)
'''


MAIN_SKELETON = '''"""Package entry point

Usage:
    python -m package [<args>...]
"""

def main(args):
    print("Hello from pyx app")

if __name__ == '__main__':
    args = docopt.docopt(__doc__)
    main(args)
'''


PROJECT_SKELETON = '''"""Project attributes

Add here all that you need to manage your project.
"""

# PyPI distrubution management.
# You can avoid typing your plain credentials here, by using keyring.
pypi = {{
    'username':      '',
    'password':      '',
    'repository':    'https://test.pypi.org/legacy'
}}
'''


ENTRY_POINT_SKELETON = '''"""main.py

Entry point for the project.
"""
import {package}


def main():
    print("Greetings from {package}!")


if __name__ == '__main__':
    main()
'''


def create_project(cwd, name):
    target = os.path.join(cwd, name)

    if not pyxutl.validate_project_name(name):
        raise pyxerr.PyxError("Name should match the following pattern: {}".format(
            pyxutl.VALID_NAME_PATTERN))

    if os.path.isdir(target):
        pyxutl.print_colors(
                "A {}/ directory already exists".format(name), 
                fg="RED",
                style="BRIGHT")
        raise pyxerr.TaskError()

    pyxutl.print_colors(
            "Creating project {}...".format(name),
            fg="GREEN",
            style="BRIGHT")

    pyxutl.mkdir(target)
    os.chdir(target)


def create_structure(name):
    pyxutl.mkdir("test")
    pyxutl.write_file(
            os.path.join("test", "test_{}.py".format(name)),
            TEST_SKELETON.format(app=pyxutl.underscores_to_camel(name)))

    pyxutl.write_file(
            ".gitignore",
            "# Write here what you want to ignore")

    pyxutl.write_file(
            "main.py",
            ENTRY_POINT_SKELETON.format(package=name))

    pyxutl.write_file(
            "README.md",
            "# {}".format(name))

    pyxutl.write_file(
            "setup.py",
            SETUP_SKELETON.format(
            project=name, user=getpass.getuser()))

    pyxutl.mkdir(name)
    pyxutl.write_file(
            os.path.join(name, "__init__.py"),
            "from importlib import import_module")

    pyxutl.mkdir(".pyx")
    pyxutl.write_file(
            os.path.join(".pyx", "project.py"),
            PROJECT_SKELETON.format(package=name))
    pyxutl.mkdir(os.path.join(".pyx", "tasks"))


def init_pipenv():
    try:
        from pipenv import core as pc
        pc.ensure_project()
    except ImportError:
        message = "pipenv is not available, "
        try:
            import pip
            message += "do you wish to install it with pip?"
            if pyxutl.prompt(message):
                pip.main(["install", "pipenv"])
                from pipenv import core as pc
                pc.ensure_project()
            else:
                pyxutl.print_warning("pipenv not found, skipping pipenv initialization")
        except ImportError:
            message += "nor is pip. Skipping pipenv initialization"
            pyxutl.print_warning(message)


def main():
    args = docopt.docopt(__doc__)

    name = args['<name>']
    cwd = os.getcwd()
    create_project(cwd, name)
    create_structure(name)
    init_pipenv()


if __name__ == '__main__':
    main()



"""Pyx project version management

A version number is something of the form x.y.z, where z could be omitted

Usage:
    version
    version upgrade
"""

import docopt
import re
from pyx import utils as pyxutl


def print_current_version():
    settings = pyxutl.get_settings()
    pyxutl.print_info("{} - Version: {}".format(settings['name'], settings['version']))


def no_command():
    print_current_version()


def upgrade():
    print_current_version()
    version = input("New version: ")
    with open("setup.py", 'r') as setup:
        content = setup.read()

    content = re.sub(r"""(\s*'version':\s*)"[^"]*",""", r"\1'{}',".format(version), content)

    with open("setup.py", 'w') as setup:
        setup.write(content)


def main():
    pyxutl.ensure_pyx()

    args = docopt.docopt(__doc__)
    if args['upgrade']:
        upgrade()
    else:
        no_command()


if __name__ == '__main__':
    main()



"""Pyx project execution

Usage:
    run
"""


import docopt
import subprocess
from pyx import utils as pyxutl


def main():
    pyxutl.ensure_pyx()
    _args = docopt.docopt(__doc__)

    subprocess.run(["pipenv", "run", "python", "main.py"])


if __name__ == '__main__':
    main()



import importlib
import os
import sys
import enum
import re
try:
    import colorama
    colorama.init()
except ImportError:
    print(
            "Module colorama was not found; "
            "you can install it with `pip3 install colorama`", 
            file=sys.stderr)
    colorama = None
from . import errors as pyxerr
from . import tasks


VALID_NAME_PATTERN = r'^[a-z][a-z0-9_]*$'


def print_colors(*args, bg="RESET", fg="RESET", style="NORMAL", **kwargs):
    if colorama is not None: 
        args = ["{}{}{}{}{}".format(
            getattr(colorama.Back, bg), 
            getattr(colorama.Fore, fg), 
            getattr(colorama.Style, style), 
            arg, 
            colorama.Style.RESET_ALL) 
            for arg in args]
    print(*args, **kwargs)


def print_error(*args, **kwargs):
    print_colors("E -", *args, fg="RED", style="BRIGHT", file=sys.stderr, **kwargs)


def print_warning(*args, **kwargs):
    print_colors("W -", *args, fg="RED", style="NORMAL", file=sys.stderr, **kwargs)


def print_info(*args, **kwargs):
    print_colors(*args, fg="GREEN", style="NORMAL")


def print_success(*args, **kwargs):
    print_colors(*args, fg="GREEN", style="BRIGHT")


def print_list(l):
    l = list(l)
    label_width = max(len(x) for x, _ in l)
    for label, text in l:
        print_colors(
                "  {label: <{width}}    ".format(label=label, width=label_width),
                fg="BLUE", style="BRIGHT", end="")
        print(text)

def prompt(message):
    while True:
        print_colors("{} [Y]es/[N]o".format(message), fg="BLUE")
        result = input("> ")
        if result in ('y', 'yes'):
            return True
        elif result in ('n', 'no'):
            return False


def load_module(path, module_name):
    """Load a module given its path, and loads it and returns it"""
    spec = importlib.util.spec_from_file_location(module_name, path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)

    return module


def mkdir(path):
    try:
        os.mkdir(path)
        print_colors("Created directory {}".format(path), fg="GREEN")

    except FileExistsError:
        print_colors("Skipped creation of directory {}".format(path), fg="RED")


def write_file(path, content):
    try:
        open(path, 'r').close()
        print_colors("Skipped writing of file {}".format(path), fg="RED")

    except FileNotFoundError:
        with open(path, 'w') as file:
            file.write(content)
        print_colors("Created file {}".format(path), fg="GREEN")


def validate_project_name(name):
    return re.match(VALID_NAME_PATTERN, name)


def underscores_to_camel(name):
    return name.replace('_', ' ').title().replace(' ', '')


def ensure_pyx():
    if not os.path.isdir(".pyx"):
        print_error("No .pyx/ directory found. Please run this inside a pyx project.")
        sys.exit(1)


def get_project():
    return load_module(os.path.join(os.getcwd(), ".pyx", "project.py"), "project")


def get_settings():
    return load_module("setup.py", "setup").settings


def is_git():
    return os.path.isdir(".git")


def get_tasks():
    tasks_dict = {}

    global_root = os.path.dirname(os.path.abspath(tasks.__file__))
    files = os.listdir(global_root)
    for f in files:
        if not f.endswith(".py") or f == "__init__.py":
            continue
        shortdoc = load_module(os.path.join(global_root, f), f[:-3]).__doc__.split('\n')[0]
        tasks_dict[f[:-3]] = shortdoc

    try:
        local_root = os.path.join(os.getcwd(), ".pyx", "tasks")
        files = os.listdir(local_root)
        for f in files:
            if not f.endswith(".py") or f == "__init__.py":
                continue
            shortdoc = load_module(os.path.join(local_root, f), f[:-3]).__doc__.split('\n')[0]
            tasks_dict[f[:-3]] = shortdoc
    except FileNotFoundError:
        pass

    return tasks_dict.items()



import setuptools

with open("PYPI_README.md", 'r') as f:
    readme = f.read()

setuptools.setup(
    name="pyx-manager",
    version="1.6.7",
    entry_points={
        'console_scripts': ["pyx = pyx.__main__:main"]
    },
    packages=setuptools.find_packages(),
    install_requires=[
        'docopt',
        'colorama',
        'twine'
        ],
    author='Armand Foucault',
    author_email='armand.foucault@telecom-bretagne.eu',
    description="Command-line project manager",
    long_description=readme,
    long_description_content_type='text/markdown',
    url="https://github.com/Afoucaul/pyx"
)



"""Run tests in a Pyx project.

Usage: 
    test
"""

import os
import docopt
import subprocess


def main(args):
    subprocess.run(["python3", "-m", "unittest", "-v", "discover", "test"])


if __name__ == '__main__':
    args = docopt.docopt(__doc__)
    main(args)
